@@ -36,9 +36,13 @@ vl_assert_almost_equal(...
   vl_binsum(Z, X, B, 3), R) ;
 
 function test_storage_classes()
-types = {@double, @single, @int64, @uint64, ...
-         @int32, @uint32, @int16, @uint16, ...
+types = {@double, @single, ...
+         @int32, @uint32, ...
+         @int16, @uint16, ...
          @int8, @uint8} ;
+if vl_matlabversion() > 71000
+  types = horzcat(types, {@int64, @uint64}) ;
+end
 for a = types
   a = a{1} ;
   for b = types
