@@ -21,9 +21,13 @@ for k=1:6
 end
 
 function test_storage_classes()
-types = {@double, @single, @int64, @uint64, ...
-         @int32, @uint32, @int16, @uint16, ...
+types = {@double, @single, ...
+         @int32, @uint32, ...
+         @int16, @uint16, ...
          @int8, @uint8} ;
+if vl_matlabversion() > 71000
+  types = horzcat(types, {@int64, @uint64}) ;
+end
 for a = types
   a = a{1} ;
   for b = types
